@@ -19,7 +19,7 @@ use Term::ANSIColor;
 use Time::HiRes qw/sleep/;
 use base qw/Exporter/;
 
-our $VERSION     = version->new('0.0.2');
+our $VERSION     = version->new('0.0.5');
 our @EXPORT_OK   = qw//;
 our %EXPORT_TAGS = ();
 
@@ -61,34 +61,52 @@ sub new {
 	$self->{background}   ||= 0;
 	$self->{sessions_max} ||= 100;
 
+	$self->{dump} = Data::Dump::Streamer->new()->Indent(4);
+
 	return $self;
 }
 
 sub read_files {
 	my ($self, @files) = @_;
-	my %files = map {$_ => undef} @files;
+	my %files = map {$_ => { name => $_ }} map { glob $_ } @files;
 	my $once = 1;
 
-	while ($self->{follow} || $once) {
-		$once = 0;
+	while ( $self->{follow} || $once == 1 ) {
+		$once++;
 		for my $file (keys %files) {
-			$files{$file} = $self->read_file($file, $files{$file});
+			# process the file for any (new) log lines
+			$self->read_file($files{$file});
 		}
 
-		sleep 0.1;
+		# every 1,000 itterations check if there are any new files matching
+		# any passed globs in, allows not having to re-run every time a new
+		# log file is created.
+		if ( $once % 1_000 ) {
+			for my $file ( map { glob $_ } @files ) {
+				# add the new file only if it doesn't already exist
+				$files{$file} ||= { name => $file };
+			}
+		}
+		else {
+			# sleep every time we have cycled through all the files to
+			# reduce CPU load.
+			sleep 0.1;
+		}
 	}
 
 	return;
 }
 
 sub read_file {
-	my ($self, $file, $fh) = @_;
+	my ($self, $file) = @_;
 
-	if (!$fh) {
+	if (!$file->{handle}) {
 		# TODO implement seeking to end and going back correct number of lines ...
-		open $fh, '<', $file or warn "Could not open $file: $!\n" and next;
+		open $file->{handle}, '<', $file->{name} or warn "Could not open $file->{name}: $!\n" and next;
 	}
+	my $fh = $file->{handle};
 
+	# read the rest of the lines in the file
 	while (my $line = <$fh>) {
 		my @line = $self->parse_line($line);
 
@@ -98,9 +116,9 @@ sub read_file {
 	}
 
 	# reset the file handle so that it can be read again;
-	seek $fh, 0, 1;
+	seek $file->{handle}, 0, 1;
 
-	return $fh;
+	return $file->{handle};
 }
 
 sub parse_line {
@@ -113,21 +131,22 @@ sub parse_line {
 		$col =~ s/\\\\/\\/g;
 	}
 
-	my $HASH1;
-	eval $log[-1];
+	# re-process the data so we can display what is needed.
+	my $DATA;
+	eval $log[-1];  ## no critic
 
-	return (@log[0..3], $HASH1);
+	return (@log[0..3], $DATA);
 }
 
-sub show_line
-{
+sub show_line {
 	my ($self, $time, $session, $level, $message, $data) = @_;
 
+	# TODO add real filtering body here
+
 	return 1;
 }
 
-sub display_line
-{
+sub display_line {
 	my ($self, $time, $session, $level, $message, $data) = @_;
 
 	my $last = $self->{last_line_time};
@@ -145,6 +164,43 @@ sub display_line
 	print color 'reset';
 	print " $level - $message\n";
 
+	if ($self->{display}) {
+		$self->display_data($data);
+	}
+
+	return;
+}
+
+sub display_data {
+	my ($self, $data) = @_;
+	my $display = $self->{display};
+
+	FIELD:
+	for my $field ( sort keys %{ $display } ) {
+		if ( ref $display->{$field} eq 'ARRAY' ) {
+			for my $sub_field ( @{ $display->{$field} } ) {
+				warn $sub_field;
+				print $self->{dump}->Names( $field . '_' . $sub_field )->Data( $data->{$field}{$sub_field} )->Out();
+			}
+		}
+		elsif ( $display->{$field} eq 0 ) {
+			next FIELD;
+		}
+		elsif ( $display->{$field} ne 1 ) {
+			$display->{$field} = [ split /,/, $display->{$field} ];
+			for my $sub_field ( @{ $display->{$field} } ) {
+				warn $sub_field;
+				print $self->{dump}->Names( $field . '_' . $sub_field )->Data( $data->{$field}{$sub_field} )->Out();
+			}
+		}
+		elsif ( $field eq 'data' && !defined $data->{data} ) {
+			# don't print anything
+		}
+		else {
+			print $self->{dump}->Names($field)->Data($data->{$field})->Out();
+		}
+	}
+
 	return;
 }
 
@@ -205,7 +261,7 @@ Log::Deep::Read - Read and prettily display log files generated by Log::Deep
 
 =head1 VERSION
 
-This documentation refers to Log::Deep::Read version 0.0.2.
+This documentation refers to Log::Deep::Read version 0.0.5.
 
 =head1 SYNOPSIS
 
@@ -265,6 +321,13 @@ Params: The data for the current log line
 
 Description: Actually prints out the line, colouring the out put as necessary.
 
+=head3 C<display_data ( $data )>
+
+Param: C<$data> - hash ref - The data to be displayed
+
+Description: Displays the log lines data based on the display rules set up
+when the object was created.
+
 =head3 C<session_colour ( $session_id )>
 
 Params: The session id that is to be coloured
